#
# Copyright 2000-2002 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms. 
#
#  @version      "@(#)wci-switch-d.prc  1.8   03/11/13 SMI"
#  @author       John Jamberdino and Jim Rayson
#  @description  WildCat Switch Proxy Agent Module Procedures
#

# This procedure derives the names of the two kinds of files (log data
# and switchName+instance data) needed by the wci-switch module.
# When invoked for deriving the latter (instanceDataFile), the second
# argument should be an empty string ("").  This routine also
# serves to consolidate all the hardwired file names into one
# routine for easier maintenance.
#
proc getFileName { type switchName } {
	set namePrefix "/var/opt/SUNWsymon/"
	switch $type {
		instanceDataFile { set nameSuffix "cfg/sfl-switch-sv.dat" }
		logFile { set nameSuffix "log/sfl-switch+$switchName.log" }
		default { set nameSuffix "UNDEFINED"; 
				  ddl print info "Bad type $type in SFL switch module filename"}
		}
	return $namePrefix$nameSuffix
	}


#
# This procedure takes the input of a string in the from of 
# ip address:agent port and returns only the ip address.
# 
proc getIPFromIPPortStr { ipportstr } {

    set portseparator :
    set sepindex [ string first $portseparator $ipportstr ]
    set res [ string range $ipportstr 0 [ incr sepindex -1 ] ]

    return $res

}


#
# This procedure writes the given information to the given wci-switch
# log based on the originating WCIX.
#
proc logTrapInfo { time desc wcix } {

    ddl print info "Log Trap: $time $desc"

	set filename [getFileName "logFile" "$wcix" ]

    set newline "$time		$desc"

    if { [ file exists $filename ] } {
	set f [ open $filename a ]
    } else {
	set f [ open $filename w+ ]
    }

    puts $f $newline

    close $f

    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip address:agent port string, sets the received property
# value in its MIB, and logs the given information.
#
proc processAndLogTrap { trap ipstr time } {

    set ip_address [ getIPFromIPPortStr $ipstr ]
    set wcix [ ip2host $ip_address ]
    set modInst [ getInstanceData $ip_address ]

    toe_csend res [ locate .contexts.$modInst.iso*$trap ] setValue 0 Received

    logTrapInfo $time "WCIX $wcix $ip_address generated Trap: $trap" $wcix
    
    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip adress-agent port string, sets the received property
# value in its MIB, and logs the given information.
#
proc setAndLogTrap { trap ipaddr modInst time } {

    set wcix [ ip2host $ipaddr ]

    toe_csend res [ locate .contexts.$modInst.iso*$trap ] setValue 0 Received
    logTrapInfo $time "WCIX $wcix $ipaddr generated Trap: $trap" $wcix
    
    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip adress-agent port string and refreshes the paroli
# portion of its MIB.
#
proc updateParoliData { trap ipstr time } {

    set ip_address [ getIPFromIPPortStr $ipstr ]
    set modInst [ getInstanceData $ip_address ]

    toe_csend res [ locate .contexts.$modInst.iso*wciSwitch.paroliUnit ] refreshValueAndTrap

    setAndLogTrap $trap $ip_address $modInst $time
    
    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip adress-agent port string and refreshes the Controller
# portion of its MIB.
#
proc updateControllerInfo { trap ipstr time } {

    set ip_address [ getIPFromIPPortStr $ipstr ]
    set modInst [ getInstanceData $ip_address ]

    toe_csend res [ locate .contexts.$modInst.iso*wciSwitch.controllerUnit ] refreshValueAndTrap

    setAndLogTrap $trap $ip_address $modInst $time
    
    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip adress-agent port string and refreshes the chassis
# portion of its MIB.
#
proc updateChassisData { trap ipstr time } {

    ddl print debug "Start updateChassisData: $trap $ipstr $time"

    set ip_address [ getIPFromIPPortStr $ipstr ]
    set modInst [ getInstanceData $ip_address ]

    ddl print debug "Middle updateChassisData: $ip_address $modInst"

    toe_csend res [ locate .contexts.$modInst.iso*wciSwitch.chassis ] refreshValueAndTrap

    setAndLogTrap $trap $ip_address $modInst $time

    ddl print debug "End updateChassisData: $trap $ip_address $modInst $time"
    
    return

}


#
# This procedure determines the instance of the wci-switch agent module
# from the given ip adress-agent port string and refreshes the all
# portions of its MIB.
#
proc updateAllData { trap ipstr time } {

    set ip_address [ getIPFromIPPortStr $ipstr ]
    set modInst [ getInstanceData $ip_address ]

    toe_csend res [ locate .contexts.$modInst.iso*wciSwitch.chassis ] refreshValueAndTrap

    toe_csend res [ locate .contexts.$modInst.iso*wciSwitch.slots ] refreshValueAndTrap

    setAndLogTrap $trap $ip_address $modInst $time
    
    return

}


#
# This procedure reads each line from the given file and returns a
# list of that data.
#
proc readAllFileData { filename } {

	set f [ open $filename r ]
	set x ""

	# Get all values from the file.
	while { [ gets $f y ] >= 0 } {
	   lappend x $y
	}

	close $f

	return $x
	
}



#
# This procedure returns a module instance from the given ip address.
# This is important for locating and operating on multi-instance objects.
#
proc getInstanceData { ipaddr } {

	set res ""
	set filename [getFileName "instanceDataFile" "" ]

	set f [ open $filename r ]

	# Read each line from the file and determine the module instance.
	while { [ gets $f line ] >= 0 } {

	   # If the ip address is in a string extract the instance name.
	   if [ string match $ipaddr* $line ] {

	      set len [ expr [ string length  $ipaddr ] + 1 ]
	      set res [ string range $line $len end ]
	      break
	
	   }

	}

	close $f

	return $res

}


###########################################################################
# The following set of procedures are part of the security effort.
# They  pull the switch
# IP address and SNMPv1 community string information out of a file
# that is created at setup time by es-setup and can only be read by
# root.  This is necessary to avoid having to store that data as
# params in the DAQ (or specified in GUI when instantiate the module)
# because then it would end up visible in files that are readable by
# anyone.  This code includes pieces cut from serengeti-platadmin-d.prc
#
# This is awkward, basically we need to get the target SC IP address
# and the appropriate SC community strings before we start, so we
# call this from a startup service....
#
# While it seems like these things would be good candidates for
# module parameters, they contain sensitive information so should
# not be exposed to the general SunMC console.
###########################################################################


# This is a utility routine that supports setparams
proc dprint { channel message } {
    ddl print $channel "SFL(wci)-switch, $message"
}

#This function reads the configuration file and returns a list of
#community strings. 
#


proc getCommunityStrings { instanceName } {
    global env

    set configfile "$env(ESDIR)/cfg/wci-switch-$instanceName-d.dat"

	dprint debug "Value for configfile=$configfile"

    if [ catch { open $configfile r } fileId ] {
        dprint error "Could not open file $configfile for reading"
        return [list public private]
    } else {
        set listtoreturn ""
        foreach line [ split [read $fileId] \n ] {
            set varname  [lindex [split $line :] 0]
            set varvalue [lindex [split $line :] 1]
            switch -exact -- $varname {
                publicCommunity {
		    set publicComm "$varvalue"
                }
                privateCommunity {
		    set privateComm "$varvalue"
                }
            } 
        }
        close $fileId
        lappend listtoreturn "$publicComm" "$privateComm" 
        return $listtoreturn
    }
        
}



#
# This routine will check the user's group membership and return the
# public or private community string accordingly
#

#WARNING: For now are just doing SRS which is
#read-only, therefore only public string is relevant.  Will do private
#string later once have "set" operations being used in the d.x and
#enhance this code accordingly

proc pickCommunityString { commStrings } {

	return [lindex $commStrings 0]
	}



#
#This function reads the configuration file and returns the IP
#address of the Switch system controller. If in the configuration file
#the host name of the Switch is stored then this function will convert
#the hostname to IP address.  This function will only return ONE IP
#address, since only one IP address should be in the config file
#

proc getRemoteIPAddress { instanceName } {
    global env
    set configfile "$env(ESDIR)/cfg/wci-switch-$instanceName-d.dat"
	
    if [ catch { open $configfile r } fileId ] {
        dprint error "Could not open file $configfile : $fileId"
        return 
    } else {
        foreach line [ split [read $fileId] \n ] {
            set varname  [lindex [split $line :] 0]
            set varvalue [lindex [split $line :] 1]
            switch -exact -- $varname {
                IPAddress {
                    if { [isIpAddress $varvalue ] } {
                        set ipAddress $varvalue
                    } else {
			set ipAddress [resolveHostName $varvalue]
                    }
                    close $fileId
                    return $ipAddress
                }
            }
        }
        close $fileId
        dprint error "Could not find Switch IP address in $configfile"
        return 
    }
}


proc setparams { instanceName } {
	# set the return code to "ok", error string to empty
    set code ok
	set x ""
    dprint info "setparams"

# Read the IP address and community string information out of the
# config file created with es-setup for the wci-switch module

	set commStrings [getCommunityStrings $instanceName]
	set publicComm [lindex $commStrings 0]
	set privateComm [lindex $commStrings 1]

	set community [ pickCommunityString $commStrings ]
	set ipAddress [getRemoteIPAddress $instanceName]

#	dprint debug "ip, public, private: $ipAddress, $publicComm, $privateComm"

# Set the community strings and IP addresses for these MIB elements
# Note that have to go down to the table level for MIB elements
# that contain tables. (i.e., most except for chassis, controllerUnit)

	set MIBelements "chassis slots.slotTable controllerUnit
		wciUnit.wciUnitTable paroliUnit.paroliTable fanUnit.fanTable 
		powerSupplyUnit.powerSupplyTable fru.fruTable 
		fruManufacturing.fruManufacturingTable
		fruInstall.fruInstallTable fruPowerEvents.fruPowerEventsTable
		fruPowerSummary.fruPowerSummaryTable fruTemperature.fruTemperatureTable 
		fruStatus.fruStatusTable fruECO.fruECOTable
		fruCustomerData.fruCustomerDataTable
		fruLaserPowerSummary.fruLaserPowerSummaryTable
		fruLaserPowerEvents.fruLaserPowerEventsTable"

    foreach i $MIBelements {
		if { [ toe_csend x [ locate .contexts.$instanceName*wciSwitch.$i ] { setParameters targetAddress $ipAddress securityName $community retries 1 timeout 60 } ] } {
			dprint error "setparams, $i setParameters failed. $x"
			set code error
		}
    }

    return -code $code
}

###########################################################################
# The following routines were added to support PV/LV (physical
# view/logical view) for the switch
###########################################################################

# Procedure to register the wci-switch module with the SunMC core hdreg 
# module.  This will make the view accessible from the "Hardware" tab of 
# the details window

proc regHdReg { instanceName } {
    dprint debug "Registering to hardware registry module for wci-switch"
    set rtn [ catch { locate *hdreg.hdreg-table } obj ]
    if { $rtn == "1" } {
	dprint error "Hardware registry module has not been loaded (needed by wci-switch)"
	return
    }
    toe_send $obj regModule wci-switch+$instanceName "PV_SFL_switch+$instanceName" S system
}


# Procedure to unregister the wci-switch module with the SunMC core 
# hdreg module.  

proc unregHdReg { instanceName } {
    dprint debug "Unregistering to hardware registry module for wci-switch"
    set rtn [ catch { locate *hdreg.hdreg-table } obj ]
    if { $rtn == "1" } {
	dprint error "Hardware registry module has not been loaded (needed by wci-switch)"
	return
    }
    toe_send $obj unregModule wci-switch+$instanceName
}

##############################################################################
# deriveAllTreeProps:

# This procedure derives all the sunSmTreeProp and sunSmNodeName for the Switch.
# It is called whenever a trap or refreshInterval leads to the "slots"
# table and all its subsidiary tables being refreshed.
# Assumes slot table and all its subordinate tables (controller, wci,
# etc) have just been refreshed and contain valid data.  Note that the
# tree structure determination starts at the slots table. 
# The heirarchy will be (for example):
#-system
#---chassis
#-----slot(0)
#-------controller (or paroli or fantray or ...)
#-----slot(3)
#-------paroli(0) (or wcix or fantray or ...)
#-----slot(n)...

# Note:  There is a SunMC limitation that prevents the sunSmTreeProp and 
# sunSmNodeName for the system node from being set it this routine.
# Instead, they are set to hardwired values in the wci-switch-d.x file

##############################################################################

proc deriveAllTreeProps { instanceName } {
	dprint debug "Starting deriveAllTreeProps"


# The sunSmTreeProp and sunSmNodeName values create internal tables.
# These tables are always indexed starting at location 1, and so if
# you want to jump directly to a slot that may not be the first slot
# in the table (e.g., because the first slot in the chassis is empty)
# then you have to pad the sunSmTreeProp table up to that point.  To
# simplify things, and because computational expense is low, just
# initialize the sunSmTreeProp and sunSmNodeName tables 
# with null stuff, and then overwrite it in the cases where
# there are table rows that are actually occupied.  This will avoid having
# to compute any padding.  Need to do this before "chassis"
# sunSmTreeProp is set because it will specify a child that requires
# indexing into the slot table.  Need to do this padding with all the
# tables that are used by PV/LV
# Don't need to init the wcix table because switch only has space for
# ONE wcix

	# Only last parameter is is quotes so that have option of putting spaces
	# in a node's name
	initNullTreeProps $instanceName wciSwitch.slots.slotTable.slotEntry "slot"
	initNullTreeProps $instanceName wciSwitch.paroliUnit.paroliTable.paroliEntry "paroli"
	initNullTreeProps $instanceName wciSwitch.fanUnit.fanTable.fanTray "fanTray"
	initNullTreeProps $instanceName wciSwitch.powerSupplyUnit.powerSupplyTable.powerSupplyEntry "powerSupply" 


# Determine which slots are occupied.  Then create a list containing
# the index numbers of only the occupied slots.  Determine if a slot
# is occupied by seeing if slotState is 2 (means "unknown" which means
# it's empty).

    set slotStateToeId [ locate .contexts.$instanceName.iso*wciSwitch.slots.slotTable.slotEntry.slotState ]
	if { [toe_csend slotStateValues $slotStateToeId getValues ] } {
		dprint error "Could not find slotState in slots table"
		return
	}
	# System controller is always installed.  It HAS to be there
	# or else the switch wouldn't be running its SNMP agent (or
	# anything else...)
	if { [llength $slotStateValues ] < 1 } {
		dprint error "slotState thinks there are NO boards (even system controller) in the chassis slotStateValues=$slotStateValues"
		return
	}
	# Even though slot numbers start at zero, want to store SunMC table
	# indices and they start at 1
	set slotRowIndex 1
	set occupiedSlotRows {}
	foreach value $slotStateValues {
		if { $value != 2 } {
			lappend occupiedSlotRows $slotRowIndex
		}
		incr slotRowIndex 1
	}
	set lastOccupiedSlotRow [ lindex $occupiedSlotRows end ]

	dprint debug "ROW numbers of the occupied slots are: $occupiedSlotRows"


# Look at the occupied slots, and derive child and brother information
# for them.  What simplifies things is that the LV display (inspired
# by serengeti) is structured to show each occupied slot and then have 
# a single child per slot.  So that alarms against the slot table are
# visible in PV, each occupied slot must have an associated
# photograph.  Therefore put the photos into the sunSmTreeProps for
# the slots rather than inside the child nodes.
# There is an assumption, when calculating child and brother indices
# for sunSmTreeProp, that all tables with > 1 rows (i.e., all but
# wcix) have been initialized (padded) with nulls.   This is done by
# the initNullTreeProps routine.
    set treeToeId [ locate .contexts.$instanceName.iso*wciSwitch.slots.slotTable.slotEntry.sunSmTreeProp ]
    set nodeNameToeId [ locate .contexts.$instanceName.iso*wciSwitch.slots.slotTable.slotEntry.sunSmNodeName ]
    set slotBoardTypeToeId [ locate .contexts.$instanceName.iso*wciSwitch.slots.slotTable.slotEntry.slotBoardType ]

# The tables whose entries are children of the slots table (e.g.,
# paroliUnit) need to have extra indices to properly accomodate
# situations where there may be boards installed that do not start at
# the first row for that table or may have empty slots in between
# occupied slots.  This allows them to index into the sunSmTreeProp
# tables correctly.
	set paroliUnitTreeIndex 1
	set wciUnitTreeIndex 1
	set fanUnitTreeIndex 1
	set powerSupplyUnitTreeIndex 1

	set occupiedListIndex 1	;# index into occupiedSlotRows, pre-incrmented by 1
	foreach slotRowIndex $occupiedSlotRows {
		if { [toe_csend slotBoardType $slotBoardTypeToeId getValue $slotRowIndex] } {
			dprint error "Could not find slotBoardType in slots table index $slotRowIndex"
			return
		}

		# NOTE: the string for the slot node names must EXACTLY
		# match one that is set in rules rwcx100 and rwcx101
		# that set in deriveAllTreeProps.  This is not elegant but
		# handles a special case with no elegant solution.

		set nodeName "Slot\([expr $slotRowIndex - 1]\)"
		switch -exact $slotBoardType {
			# "empty"
			1 { 
				dprint debug "slotBoardType of 1 = empty should never occur"
			}
			# "controller"
			2 {
				set child "wciSwitch.controllerUnit"
				set childIndex 1
				if { $slotRowIndex != $lastOccupiedSlotRow } {
					set brother "wciSwitch.slots"
					set brotherIndex [ lindex $occupiedSlotRows $occupiedListIndex ]
				} else {
					set brother "null"
					set brotherIndex 0
				}
				set photoName "controller"

				# set sunSmTreeProps and sunSmNodeName for the child board
				set leafLocation $child
				set leafRowIndex 0	;# controller not a table so row index = 0
				set leafNodeName "Controller"
				deriveLeafTreeProp $instanceName $leafLocation $leafRowIndex $leafNodeName
			}

			# "wciUnit"
			3 {
				set child "wciSwitch.wciUnit"
				set childIndex $wciUnitTreeIndex
				if { $slotRowIndex != $lastOccupiedSlotRow } {
					set brother "wciSwitch.slots"
					set brotherIndex [ lindex $occupiedSlotRows $occupiedListIndex ]
				} else {
					set brother "null"
					set brotherIndex 0
				}
				set photoName "wcix"

				# set sunSmTreeProps and sunSmNodeName for the child board
				set leafLocation "$child.wciUnitTable.wciEntry"
				set leafRowIndex $childIndex
				set leafNodeName "SFLX\([expr $slotRowIndex - 1]\)"
				deriveLeafTreeProp  $instanceName $leafLocation $leafRowIndex $leafNodeName
				set wciUnitTreeIndex [ expr $wciUnitTreeIndex + 1 ]
			}


			# "paroliUnit"
			4 {
				set child "wciSwitch.paroliUnit"
				set childIndex $paroliUnitTreeIndex
				if { $slotRowIndex != $lastOccupiedSlotRow } {
					set brother "wciSwitch.slots"
					set brotherIndex [ lindex $occupiedSlotRows $occupiedListIndex ]
				} else {
					set brother "null"
					set brotherIndex 0
				}
				set photoName "paroli"

				# set sunSmTreeProps and sunSmNodeName for the child board
				set leafLocation "$child.paroliTable.paroliEntry"
				set leafRowIndex $childIndex
				set leafNodeName "Paroli\([expr $slotRowIndex - 1]\)"
				deriveLeafTreeProp  $instanceName $leafLocation $leafRowIndex $leafNodeName 
				set paroliUnitTreeIndex [ expr $paroliUnitTreeIndex + 1 ]
			}


			# "fanUnit"
			5 {
				set child "wciSwitch.fanUnit"
				set childIndex $fanUnitTreeIndex
				if { $slotRowIndex != $lastOccupiedSlotRow } {
					set brother "wciSwitch.slots"
					set brotherIndex [ lindex $occupiedSlotRows $occupiedListIndex ]
				} else {
					set brother "null"
					set brotherIndex 0
				}
				set photoName "fantray"

				# set sunSmTreeProps and sunSmNodeName for the child board
				set leafLocation "$child.fanTable.fanTray"
				set leafRowIndex $childIndex
				set leafNodeName "Fan Tray\([expr $slotRowIndex - 1]\)"
				deriveLeafTreeProp  $instanceName $leafLocation $leafRowIndex $leafNodeName
				set fanUnitTreeIndex [ expr $fanUnitTreeIndex + 1 ]
			}


			# "powerSupplyUnit"
			6 {
				set child "wciSwitch.powerSupplyUnit"
				set childIndex $powerSupplyUnitTreeIndex
				if { $slotRowIndex != $lastOccupiedSlotRow } {
					set brother "wciSwitch.slots"
					set brotherIndex [ lindex $occupiedSlotRows $occupiedListIndex ]
				} else {
					set brother "null"
					set brotherIndex 0
				}
				set photoName "powersupply"

				# set sunSmTreeProps and sunSmNodeName for the child board
				set leafLocation "$child.powerSupplyTable.powerSupplyEntry"
				set leafRowIndex $childIndex
				set leafNodeName "Power Supply\([expr $slotRowIndex - 1]\)"
				deriveLeafTreeProp  $instanceName $leafLocation $leafRowIndex $leafNodeName
				set powerSupplyUnitTreeIndex [ expr $powerSupplyUnitTreeIndex + 1 ]
			}
		}

		# Set sunSmTreeProps and sunSmNodeName for this slot
		# The photo locations in the pvimages .loc file are all of the
		# form slot-1, slot-2, etc.  Setting of photoView and
		# photoLocation could be done outside this loop to save a few
		# cycles, but more readable to leave them here.
		# NOTE: don't put ANY whitespace inside the treeProp or it
		# will become part of the field names.
		if { $childIndex < 1 || $brotherIndex < 0 } {
			dprint error "child index must be > 0 and brother index > 1 : in $nodeName childIndex=$childIndex, brotherIndex=$brotherIndex"
			return
		}
		set photoLocation "slot-[expr $slotRowIndex - 1]"
		set photoView "rear"
		set treeProp "{$nodeName:$child:$brother:$childIndex:$brotherIndex:$photoName:$photoLocation:$photoView:null:null:null}" 
		if { [toe_csend errorMsg $treeToeId setValue $slotRowIndex $treeProp ] } {
			dprint error "Could not find sunSmTreeProp for slots table row $rowindex: $errorMsg"
			return
		}
		if { [toe_csend errorMsg $nodeNameToeId setValue $slotRowIndex "{$nodeName}"] } {
			dprint error "Could not find sunSmNodeName for slots table row $rowindex: $errorMsg"
			return
		}

		incr occupiedListIndex 1
	}
	# The sunSmTreeProp and sunSmNodeName for the "system" node are
	# hardwired and specified in the wci-switch-d.x file, due to some
	# SunMC requirements.  The above processing set the  sunSmTreeProp 
	# and sunSmNodeName values for the "slots" table and the individual
	# board contained within each slot.  Now need to set them for the
	# "chassis".  Its child is the first occupied slot in the "slots"
	# table.

    set treeToeId [ locate .contexts.$instanceName.iso*wciSwitch.chassis.sunSmTreeProp ]
    set nodeNameToeId [ locate .contexts.$instanceName.iso*wciSwitch.chassis.sunSmNodeName ]

	set chassisRowIndex 0
	set child "wciSwitch.slots"
	set childIndex [ lindex $occupiedSlotRows 0 ]
	set nodeName "Chassis"
	set brother "null"
	set brotherIndex 0
	set photoName "chassis"
	set photoLocation "chassis-origin"
	set photoView "rear"
	set treeProp "{$nodeName:$child:$brother:$childIndex:$brotherIndex:$photoName:$photoLocation:$photoView:null:null:null}" 

	if { [toe_csend errorMsg $treeToeId setValue $chassisRowIndex $treeProp] } {
		dprint error "Could not find sunSmTreeProp for chassis row=$chassisRowIndex: $errorMsg"
		return
	}
	if { [toe_csend errorMsg $nodeNameToeId setValue $chassisRowIndex "{$nodeName}"] } {
		dprint error "Could not find sunSmNodeName for chassis row=$chassisRowIndex: $errorMsg"
		return
	}

	dprint debug "Set chassis sunSmTreeProp row=$chassisRowIndex to: $treeProp"

# Have to send a refresh trap to system.total_config_changes whenever
# sunSmTreeProps is updated so that PV/LV will read the new values
# So, do that as the last thing whenenve deriveTreeProps is run
	if { [toe_csend errorMsg [ locate .contexts.$instanceName.iso*wci*wci.switch.system.total_config_changes ] sendRefreshTrap] } {
		dprint error "Could not refresh total_config_changes: $errorMsg"
		return
	}
}


##############################################################################
# deriveLeafTreeProp:
#
# Derives the sunSmTreeProp and sunSmNodeName for a single board, with
# no children or siblings.  This is exactly what is needed for each
# individual occupied slot in the "slots" table, and these boards
# serve as leaves in the tree hierarchy.
##############################################################################

proc deriveLeafTreeProp { instanceName location rowIndex nodeName } {

	dprint debug "Setting leafNode (single board) sunSmTreeProp: instance=$instanceName, location=$location, rowIndex=$rowIndex, nodeName=$nodeName" 

	# Derive a location name that matches contents of .loc file that
	# accompanies the photographs
    set tableToeId [ locate .contexts.$instanceName.iso*$location ]
    set nodeNameToeId [ locate .contexts.$instanceName.iso*$location.sunSmNodeName ]
    set treeToeId [ locate .contexts.$instanceName.iso*$location.sunSmTreeProp ]

	set treeProp "{$nodeName:null:null:0:0:-:null:null:null:null:null}" 
	if { [toe_csend errorMsg $treeToeId setValue $rowIndex $treeProp ] } {
		dprint error "Could not find sunSmTreeProp for slots table row $rowindex: $errorMsg"
		return
	}
	if { [toe_csend errorMsg $nodeNameToeId setValue $rowIndex "{$nodeName}"] } {
		dprint error "Could not find sunSmNodeName for slots table row $rowindex: $errorMsg"
		return
	}
}


##############################################################################
# initNullTreeProps:
#
# This is called by deriveTreeProps to initialize the 
# sunSmTreeProp and sunSmNodeName properties for tables to some
# neutral null value.  It does it up to the maximum occupied row
# in the table.  If a table is empty (e.g., because no paroli
# cards are installed in the chassis) , it just returns 
# without doing anything.
##############################################################################

proc initNullTreeProps { instanceName location name} {

	dprint debug "Init sunSmTreeProp to null: instance=$instanceName, location=$location, name=$name"


    set tableToeId [ locate .contexts.$instanceName.iso*$location ]
	set qty [toe_send $tableToeId getTableDepth]
	dprint debug "Init sunSmTreeProp inside table $location to depth $qty"
    if { $qty == 0 } {
		return
	}

    set treeToeId [ locate .contexts.$instanceName.iso*$location.sunSmTreeProp ]
    set nodeNameToeId [ locate .contexts.$instanceName.iso*$location.sunSmNodeName ]
	# Remember that SunMC uses 1 (not zero) as first index into a
	# table
	for { set rowIndex 1 } { $rowIndex <= $qty } { incr rowIndex 1 } {
		set nodeName "$name\([ expr $rowIndex - 1 ]\)"
		set treeProp "{$nodeName:null:null:null:null:-:null:null:null:null:null}"
		if { [toe_csend errorMsg $treeToeId setValue $rowIndex $treeProp ] } {
			dprint error "Could not find and init sunSmTreeProp for table $location row $rowindex: $errorMsg"
			return
		}
		if { [toe_csend errorMsg $nodeNameToeId setValue $rowIndex "{$nodeName}"] } {
			dprint error "Could not find and init sunSmNodeName for table $location row $rowindex: $errorMsg"
			return
		}


	}

}

