#
# Copyright (c) %G% by Sun Microsystems, Inc. All rights reserved.
#
#  @version      "%Z%%M%  %I%   %E% SMI"
#  @author       John Jamberdino
#  @description  WildCat Fabric Configuration Module
#

#
# This procedure takes the input of a filename and an index into the file
# and if the file exits returns the value at the index.  This procedure
# will create the file if it does not exist and returns a default value.
#
proc getFmConfData { filename index } {

	set result ""

	#
	# If the file exists then read the desired value otherwise
	# create and write to the file default values.
	#
	if { [ file exists $filename ] } {

	   set f [ open $filename r ]

	   for {set i 0} {$i < $index} {incr i 1} {
	      gets $f result
	   }
	   
	} else {

	   set f [ open $filename w+ ]
	   puts $f "none"
	   puts $f "localhost"
	   puts $f "6099"
	   puts $f "0"

	   set result "0"

	}

	close $f

	# Set the read value.
	setValue 0 $result

}


#
# This procedure takes the input of a filename, an index into the file,
# and a value to write to the calling property and a file at that index
# if the file exists.
#
proc setFmConfData { filename index value } {

	set x ""

	# Read from only an existing file.
	if { [ file exists $filename ] } {

	   set f [ open $filename r ]

	   # Get all values from the file.
	   while { [ gets $f y ] >= 0 } {
	      lappend x $y
	   }

	   close $f

	   set len [ llength $x ]
	
	   # Open the file to write the old values and the new value.
	   set f [ open $filename w+ ]

	   for {set i 1} {$i <= $len} {incr i 1} {

	      if { $i == $index } {
	         puts $f $value
	      } else {
		 set j $i
		 puts $f [ lindex $x [ incr j -1 ] ]
	      }

	   }

	   close $f
	   
	} 

	# Set the read value.
	setValue 0 $value

}

#
# This procedure will update the modification time of the FM XML file
# with the current modification time.  It will also
# store a new time to the given file at a specified index. 
#
proc getAndSetFmXMLFileModTime { filename index } {

	set filepath [ valueOf fm.properties.fabricXMLFilePath.0 ]
	set name [ valueOf fm.properties.name.0 ]
	set newtime 0

	# Get the modification data on an existing file only.
	if { [ file exists "$filepath$name.xml" ] } {
	   set newtime [ file mtime "$filepath$name.xml" ]
	   setFmConfData $filename $index $newtime
	   toe_send [ locate fm.properties.fabricXMLFileModStatus ] setValue 0 "Unmodified"
	}

	return $newtime

}

#
# This procedure will compare the modification time of the FM XML file
# stored in the MIB to the current modification time.  If the times are the 
# same this procedure will return an unmodified status, otherwise the new 
# time and a modified status will be written to the MIB.  It will also
# store a new time to the given file at a specified index. 
#
proc getFmXMLFileStatus { filename index } {

	set result "Unmodified"

	set filepath [ valueOf fm.properties.fabricXMLFilePath.0 ]
	set name [ valueOf fm.properties.name.0 ]

	# Get the modification data on an existing file only.
	if { [ file exists "$filepath$name.xml" ] } {

	   set oldtime [ valueOf fm.properties.fabricXMLFileModTime.0 ]
	   set newtime [ file mtime "$filepath$name.xml" ]

 	   #
    	   # If the mod time has changed inform the module and reset
	   # the file value.
	   #
	   if { $oldtime != $newtime } {

	      set result "Modified"
	      toe_send [ locate fm.properties.fabricXMLFileModTime ] setValue 0 $newtime
	      setFmConfData $filename $index $newtime

	   }
	   
   	}

	return $result

}

#
# This procedure wil return the agent port as specified in the
# domain-config.x file.
#
proc getAgentPort { } {

    # If the agent port is not found in the SunMC file, return the default.
    set result 161
    set count 0
    set found 0

    # Open the domain-config.x to parse for the agent port.
    set f [ open /var/opt/SUNWsymon/cfg/domain-config.x r ]
        
    #
    # Continue to search for the agent port until the agent string is found
    # or the max count is reached.
    #
    while { ($found == 0) && ($count < 30) } {

	gets $f dline

	if { [ string match *agent* $dline ] } {

	   set incnt 0

	   while { ($found == 0) && ($incnt < 10) } {

		gets $f dline

		if { [ string match *snmpPort* $dline ] } {

		   regsub "snmpPort = " $dline "" tmpres
		   set result [ string trim $tmpres ]
		   echo $result
		   incr found

		}

		incr incnt

	   }
	   	   
	}

	incr count

    }

    close $f
    setValue 0 $result

}

#
# This procedure returns the name of the discovery object table 
# in order for the creation of a wildCat composite object.
#
proc getWCIDOTName { } {

   set resName none
   set DOTNames { one two three four five six seven eight nine ten }
   set index 1

   #
   # Determine the name of the DOT which has no current information
   # stored in it.
   #
   foreach name $DOTNames {

	set depth [ toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.discovery_$name.discoveryObjectTable.discoveryObjectTableEntry ] getTableDepth ]

	if { $depth == 0 } {

	   set resName discovery_$name
	   toe_send [ locate fmDiscovery.discoveryTableNum ] setValue 0 $index
	   break

	}

	incr index

   }

   return $resName

}


#
# This procedure will cause the deletion of the given DOT row data.
#
proc deleteDOTEntries { tablename } {

	toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.magic_number ] setValue 0 "53444f54"

}


#
# This procedure returns the instance of a WCI Switch Agent Module.
#
proc getSwitchInstance { ipaddr } {

	set res ""
	set filename "/var/opt/SUNWsymon/cfg/sfl-switch-sv.dat"

	if { [ file exists $filename ] } { 
	
 	    set f [ open $filename r ]

	    # Read each line from the file and determine the module instance.
	    while { [ gets $f line ] >= 0 } {

	       if [ string match $ipaddr* $line ] {

	          set len [ expr [ string length  $ipaddr ] + 1 ]
	          set res [ string range $line $len end ]
	          break
	
	       }

	    }

	    close $f

	}

	return $res

}


#
# This procedure returns the agent port of the given host as specified
# in the WCI FM XML File and contained in the MIB.
#
proc getXMLAgentPort { hostname } {
	
	set resport 161

	# Get the member names from the member table.
	set membernames [ toe_send [ locate fm.fabricInfo.memberTable*memberName ] getValues ]

	# If the member is in the table find the corresponding port.
	set index [ lsearch -exact $membernames $hostname ]

	if { $index != -1 } {

		set agentports [ toe_send [ locate fm.fabricInfo.memberTable*agentPort ] getValues ]
		set resport [ lindex $agentports $index ]

	}

	return $resport

}


#
# This procedure returns the name of the discovery object table 
# and writes the necessary data to that table in order for the creation
# of a wildCat composite object.
#
proc refreshDiscoveryObjectTable { } {

    set index       0
    set hstname     [ info hostname ]
    set ip_address  [ resolveHostName $hstname ]
    set returnlist  ""

    # Get all needed MIB values.
    set port [ valueOf fm.properties.agentPort.0 ]
    set fname [ valueOf fm.properties.name.0 ]
    set tablename [ valueOf fmDiscovery.discoveryTableName.0 ]
    set modinst [ valueOf fm.properties.modInstance.0 ]

    #
    # Get the DOT name if it has not been found, otherwise delete the DOTs
    # data.
    #
    if { $tablename == "none" } {
	set tablename [ getWCIDOTName ]
    } else {
	deleteDOTEntries $tablename
    }

    # Do not continue if there are no tables to be written to.
    if { $tablename == "none" } {
	ddl print info "wci-fmconf: Too many instances detected, no"
	ddl print info "wci-fmconf: composite object support is available."
	return $tablename
    }

    # Get the DOT number.
    set tableNum [ valueOf fmDiscovery.discoveryTableNum.0 ]

    # Setup the DOT data for the composite object.
    set row [ list [ incr index ] "$hstname WCI_Composite_Object$tableNum" \
      "" 2 $ip_address $port \
      "wci-composite-object-type$tableNum" "Fire Link Interconnect - $fname" \
      "WCI Composite Object $tableNum" ]

    # Write the object information to the WCI DOT.
    toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

    # Setup the DOT data for WCI FM object.
    set row [ list [ incr index ] "WCI_FM_$ip_address" \
      "$hstname WCI_Composite_Object$tableNum" 5 $ip_address $port \
      "wci-fm-type" "Fabric - $fname" \
      "wci-fmconf+$modinst" ]

    # Write the object information to the WCI DOT.
    toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

    # Setup the DOT data for the FM Host.
    set row [ list [ incr index ] "FM_Host_$ip_address" \
      "$hstname WCI_Composite_Object$tableNum" 1 $ip_address $port \
      "" "FM Host - $hstname" \
      "Fabric Manager Host" ]

    # Write the object information to the WCI DOT.
    toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

    # Read all fabric member names from the MIB.
    set membernames [ toe_send [ locate fm.fabricInfo.memberTable*memberName ] getValues ]

    set memberindex 1

    # Setup the DOT data for all members of the fabric.
    foreach name $membernames {

	#
	# If the name is not N/A then this is an RSM node, otherwise
	# determine if the member is an SSM Node or a switch.
	#
	if { $name != "N/A" } {

	   # If the RSM Node is already in the list, do not add it.
	   if { [ lsearch -exact $returnlist $name ] == -1 } {

	      # Ensure that the hostname given provides a valid IP.
	      if { [ catch {resolveHostName $name} ipadd ] } { 

		 ddl print info "wci-fmconf: IP Address for $name not found\n"
		 continue

	      } else {

		 set aport [ getXMLAgentPort $name ]

	         set row [ list [ incr index ] "WCI-Node-$name" \
                   "$hstname WCI_Composite_Object$tableNum" 1 $ipadd $aport \
                   "" "$name" \
                   "Fabric Member Node" ]

	         # Write the object information to the WCI DOT.
    		 toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

	      }

	   }

	} else {

	   set scname [ toe_send [ locate fm.fabricInfo.memberTable*scHostName ] getValue $memberindex ]
	   set domainname [ toe_send [ locate fm.fabricInfo.memberTable*domainID ] getValue $memberindex ]

	   if { $domainname != "N/A" } {

		#
		# Determine if the SSM Node is already in the list, if it is
		# then do not duplicate, otherwise add its information.
		#
		if { [ lsearch -exact $returnlist $scname-$domainname ] == -1 } {

		   # Ensure that the hostname given provides a valid IP.
		   if { [ catch {resolveHostName $scname-$domainname} ipadd ] } { 

		      ddl print info "wci-fmconf: IP Address for $scname-$domainname not found\n"
		      continue

	      	   } else {

		      set aport [ getXMLAgentPort $scname-$domainname ]

		      set row [ list [ incr index ] \
		         "WCI-Node-$scname-$domainname" \
         	         "$hstname WCI_Composite_Object$tableNum" 1 $ipadd $aport \
         	         "" "$scname-$domainname" \
         	         "Fabric Member Node" ]

		      # Write the object information to the WCI DOT.
    		      toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

		  }

		}

	   } else {
		
		#
		# Determine if the switch is already in the list, if it is
		# then do not duplicate, otherwise add its information.
		#
		if { [ lsearch -exact $returnlist $scname ] == -1 } {

		   # Ensure that the hostname given provides a valid IP.
		   if { [ catch {resolveHostName $scname} ipadd ] } { 

		      ddl print info "wci-fmconf: IP Address for $scname not found\n"
		      continue

	      	   } else {

		      # Get the switch module instance for the switch object.
		      set smodinst [ getSwitchInstance $ipadd ]

		      if { [ string compare "" $smodinst ] != 0 } {

			  set aport [ getXMLAgentPort $scname ]

	   	      	  set row [ list [ incr index ] "WCI-Switch-$scname" \
        	           "$hstname WCI_Composite_Object$tableNum" 5 $ip_address $aport \
         	           "wci-switch-type" "$scname - Switch" \
         	           "wci-switch+$smodinst" ]

		      	  # Write the object information to the WCI DOT.
    		      	  toe_send [ locate .iso.org.dod.internet.private.enterprises.sun.prod.wci.agent.modules.serverSupport.wci.dot.$tablename.discoveryObjectTable.discoveryObjectTableEntry ] setValue $index $row

		      } else {
			ddl print info "WCI-FMCONF: No WCI-Switch Module found for Switch $scname"
		      }

		   }

		}

	   }

	}

	incr memberindex

    }

    return $tablename

}
